% Constructor of the finite elements block class.
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options:
% mater =material,
% gcells = cell array of geometry cells,
% integration_rule= integration rule object
% Rm= transformation matrix, in columns are the basis vectors expressed in
%   the global Cartesian basis.
%
function retobj = feblock (varargin)
    class_name='feblock';
    parent=classbase;
    if (nargin == 0)
        self.mater = [];
        self.gcells = [];
        self.integration_rule = [];
        self.Rm=[];
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            self.mater = options.mater;
            self.gcells = options.gcells;
            if (length(self.gcells) < 1)
                error('Need at least one gcell!');
            end
            for i=1:length(self.gcells)
                if (~isa(self.gcells(i),class(self.gcells(1))))
                    error ('All gcells must be of the same class!');
                end
            end
            self.integration_rule = options.integration_rule;
            Rm=[];
            if isfield(options,'Rm')
                Rm= options.Rm;
                if strcmp(class(Rm),'double')
                    if norm(Rm'*Rm-eye(size(Rm, 2)))>1e-9
                        error('Non-orthogonal local basis matrix!');
                    end
                elseif strcmp(class(Rm),'function_handle')
                else
                    error(['Cannot handle class of Rm: ' class(Rm)])
                end
            end
            self.Rm=Rm;
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end

