% Measure the discrete manifold. In other words, integrate some scalar
% function over the geometric cells. When the scalar function returns just
% +1, this function measures the volume (number of points, length, area,
% 3-D volume, according to the manifold dimension). When the function
% returns the mass density, the method measures the mass, and so on.
%
% Input
% self = feblock 
% geom = geometry field
% fh   = optional function handle; when not supplied, function 
%         returning +1 is assumed;
%
function result = measure (self, geom, varargin)
    gcells =self.gcells;
    ngcells = length(self.gcells);
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    if nargin >=3
        fh =varargin{1};
    else
        fh =@(x) (1);
    end
    result = 0;
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = get(gcells(i), 'conn'); % connectivity
        x = gather(geom, conn, 'values', 'noreshape');
        % Loop over all integration points
        for j=1:npts_per_gcell
            N = Nmat(gcells(i), pc(j,:));
            Nder = Ndermat_param (gcells(i), pc(j,:));
            detJ = Jacobian(gcells(i),pc(j,:),x'*Nder);
            result = result ...
                + fh(map_to_xyz(gcells(i),pc,x))*detJ*w(j);
        end
    end
end
