%%  Define some auxiliary variables
E=100;
nu=0.499;
integration_order=1;
W= 20;
H=5;
L= 100;
htol=min([L,H,W])/1000;
magn = 0.05;
graphics = ~true;

mf =@t4block;
blf =@feblock_defor_ss;
ir=tet_rule(1);
fir =tri_rule(1);
styl='rd--';

% mf =@t4block;
% blf =@feblock_defor_ss_avg_b_t4;
% integration_order=1;
% styl='ks-';

% mf =@block;
% blf =@feblock_defor_ss_sri;
% integration_order=[1, 2];
% styl='bo-.';

mult=1;
ns=[2, 4, 6];

% mult=2;
% ns=[1, 2, 4];

% mult=3;
% ns=[1, 2, 4];

% mult=4;
% ns=[1, 2, 4];
% 
neqns=[];
se=[];

%% Create the mesh and initialize the geometry

% for n=[3]
for n=ns
    [fens,gcells]= mf(W, L, H,mult*n,2*mult*n,n);
    %         mesh{1}=fens;
    %         mesh{2}=gcells;
    %         drawmesh(mesh,'shrink', 0.9,'facecolor','red'); view (2); pause
    mater = mater_defor_ss_linel_iso_triax (struct('E',E,'nu',nu));
    feb = blf(struct ('mater',mater, 'gcells',gcells,...
        'integration_rule',ir));
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));

    %% Define the displacement field, and zero it out
    u   = 0*geom;

    %% Apply the EBC''s
    ebc_fenids=fenode_select(fens,struct('box',[0 W 0 0 0 H]));
    ebc_prescribed=ones(length(ebc_fenids),1);
    ebc_comp=[];
    ebc_val=zeros(length(ebc_fenids),1);
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=fenode_select(fens,struct('box',[W W 0 L 0 H]));
    ebc_prescribed=ones(length(ebc_fenids),1);
    ebc_comp=ebc_fenids*0+1;
    ebc_val=zeros(length(ebc_fenids),1);
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    u   = apply_ebc (u);

    %% Number the equations
    u   = numbereqns (u);
    disp (['n=' num2str(n) ', number of equations =' num2str(get(u,'neqns'))])

    %% Assemble the stiffness matrix
    K = start (sparse_sysmat, get(u, 'neqns'));
    ems = stiffness(feb, geom, u);
    K = assemble (K, ems);
    K = finish (K);

    %% Assemble the load vector
    fi = force_intensity(struct('magn',[0;0; -magn]));
    bdry_gcells = mesh_bdry(gcells, []);
    bcl = gcell_select(fens, bdry_gcells, ...
        struct ('box',[0 W L L 0 H],'inflate',htol));
    lfeb = feblock_defor_ss(struct ('mater',mater, 'gcells',bdry_gcells(bcl),...
        'integration_rule',fir));
    F = start (sysvec, get(u, 'neqns'));
    F = assemble (F, distrib_loads(lfeb, geom, u, fi));
    F = finish(F);

    %% Solve K*u=F
    b=get(F,'vec'); 
    a=get(K,'mat');
    %     x = bicg(get(K,'mat'),get(F,'vec'), 1e-6, 14000, diag(diag (get(K,'mat'))));
    %     x=0*b;
    %     for it = 1:50
    %         res=b-a*x;
    %         norm (res)
    %         x= x+a\res;
    %     end
    [Lf,Uf]=lu(a);
    x=Uf\(Lf\b);

    %% Transfer the solution to the field u
    u = scatter_sysvec(u, x);
    disp (['   Strain energy =' num2str(x'*b/2)])
    neqns=[neqns get(u,'neqns')];
    se =[se x'*b/2];

    %% Plot the displacement field u
    if graphics
        gv=graphic_viewer;
        gv=reset (gv,[]);
        scale=1;
        % draw(feb,gv, struct ('x', geom,'u',0*u, 'facecolor','none'));
        fld = field_from_integration_points(feb, geom, u, [], 'Cauchy', 2);
        nvals=get(fld,'values');
        dcm=data_colormap(struct ('range',[min(nvals),max(nvals)], 'colormap',jet));
        colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
        draw(feb, gv, struct ('x',geom,'u', scale*u, 'colorfield',colorfield, 'shrink',1));
        colormap(jet);
        cbh=colorbar;
        set(cbh,...
            'Position',[0.8 0.15 0.1 0.7],...
            'YLim',[0,1],...
            'YTick',[0,1],...
            'YTickLabel',{[num2str(min(nvals))],[num2str(max(nvals))]});%{[num2str((min(nvals)))],[num2str((max(nvals)))]}
        set(get(cbh,'XLabel'),'String','\sigma_{y}');
        title ('Stress');
    end
end
plot(neqns,se,styl,'linewidth',mult*3); hold on
figure (gcf)